# (c) Copyright 2019 Trent Hauck
# All Rights Reserved
"""Stores the alphabets, these are the same as BioPython."""

# pylint: disable=invalid-name

protein_letters = "ACDEFGHIKLMNPQRSTVWY"
extended_protein_letters = "ACDEFGHIKLMNPQRSTVWYBXZJUO"
ambiguous_dna_letters = "GATCRYWSMKHBVDN"

unambiguous_dna_letters = "GATC"
extended_dna_letters = "GATCBDSW"

ambiguous_rna_letters = "GAUCRYWSMKHBVDN"
unambiguous_rna_letters = "GAUC"
