# (c) Copyright 2018 Trent Hauck
# All Rights Reserved
"""Top-level GCGC module."""

__version__ = "0.11.0"

from gcgc.tokenizer import SequenceTokenizer
from gcgc.tokenizer import SequenceTokenizerSpec
